;
;    ICON APPLICATION
;
;    Compile with FASM for Menuet
;
   
use32
   
                org     0x0
   
                db      'MENUET00'              ; 8 byte id
                dd      38                      ; required os
                dd      START                   ; program start
                dd      I_END                   ; program image size
                dd      16384                   ; required amount of memory
                                                ; esp = 0x7FFF0
                dd      I_PARAM
   
   
; params 4 xpos 4 ypos 11 iconfile 11 startfile 7 label
   
   
START:                          ; start of execution
   
    mov  esp,16300
   
    cmp  [I_PARAM],byte 0
    jne  nohalt
    mov  eax,-1
    int  0x40
  nohalt:
   
    mov  eax,[I_PARAM+0]
    sub  eax,0x01010101
    mov  [xpos],eax
    mov  eax,[I_PARAM+4]
    sub  eax,0x01010101
    mov  [ypos],eax
   
    mov  esi,I_PARAM+8
    mov  edi,fname
    mov  ecx,11
    cld
    rep  movsb
   
    mov  esi,I_PARAM+8+11
    mov  edi,start_file
    mov  ecx,11
    cld
    rep  movsb
   
    mov  esi,I_PARAM+8+11+11
    mov  edi,labelt
    mov  ecx,7
    cld
    rep  movsb
   
    mov  eax,40                 ; get also event background change
    mov  ebx,1+2+4+16
    int  0x40
   
    call draw_window
   
   
still:
   
    mov  eax,10                 ; wait here for event
    int  0x40
   
    cmp  eax,1                  ; redraw request ?
    jz   red
    cmp  eax,2                  ; key in buffer ?
    jz   key
    cmp  eax,3                  ; button in buffer ?
    jz   button
    cmp  eax,5                  ; background redraw ?
    jz   check
   
    jmp  still
   
  check:
    call check_icon
    jmp  still
   
  red:                          ; redraw
    call draw_window
    jmp  still
   
  key:                          ; key
    mov  eax,2                  ; just read it and ignore
    int  0x40
    jmp  still
   
  button:                       ; button
    mov  eax,17                 ; get id
    int  0x40
   
    cmp  ah,1                   ; button id=1 ?
    jnz  noid1
    mov  eax,19
    mov  ebx,start_file
    int  0x40
  noid1:
   
    jmp  still
   
   
csum   dd  0x0
   
dtype  dd  0x0
   
   
check_icon:
   
     pusha
   
     call draw_icon
   
     popa
     ret
   
   
   
   
;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************
   
   
draw_window:
   
    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,1                     ; 1, start of draw
    int  0x40
   
    mov  eax,14
    int  0x40
   
    sub  eax,60*65536
    mov  ebx,eax
    mov  bx,40
   
                                   ; DRAW WINDOW
    mov  eax,0                     ; function 0 : define and draw window
    mov  ebx,[xpos-2]
    mov  ecx,[ypos-2]
    mov  bx,41                     ; [x start] *65536 + [x size]
    mov  cx,50                     ; [y start] *65536 + [y size]
    mov  edx,0x00000000            ; color of work area RRGGBB,8->color gl
    mov  esi,0x00000000            ; color of grab bar  RRGGBB,8->color gl
    mov  edi,0x00000000            ; color of frames    RRGGBB
    int  0x40
   
    mov  eax,8    ; button
    mov  ebx,40
    mov  ecx,50
    mov  edx,1
    mov  esi,0
    int  0x40
   
    call draw_icon
   
    mov  eax,12
    mov  ebx,2
    int  0x40
   
    ret
   
   
draw_icon:
   
    mov  eax,14
    int  0x40
    add  eax,0x00010001
    mov  [scrxy],eax
   
    mov  eax,39
    mov  ebx,4
    int  0x40
    mov  [bgrdrawtype],eax
   
    mov  eax,39     ; get background size
    mov  ebx,1
    int  0x40
    mov  [bgrxy],eax
   
    mov  eax,6
    mov  ebx,fname
    mov  ecx,0
    mov  edx,0xffffffff
    mov  esi,I_END+256
    int  0x40
   
    mov  ebx,41
    mov  ecx,0
    mov  esi,I_END+256+54+32*3*33-3
   
  newb:
   
    push ebx
    push ecx
   
    cmp  ebx,4
    jb   yesbpix
    cmp  ebx,36
    jge  yesbpix
    cmp  ecx,32
    jg   yesbpix
   
   
    mov  eax,[esi]
    sub  esi,3
    and  eax,0xffffff
    cmp  eax,0
    jnz  nobpix
   
   
  yesbpix:
   
    cmp   [bgrdrawtype],dword 2
    jne   nostretch
   
    mov   eax,[ypos]
    add   eax,ecx
    xor   edx,edx
    movzx ebx,word [bgrxy]
    mul   ebx
    xor   edx,edx
    movzx ebx,word [scrxy]
    div   ebx
    xor   edx,edx
    movzx ebx,word [bgrxy+2]
    mul   ebx
    push  eax
   
    mov   eax,[xpos]
    add   eax,[esp+8]
    xor   edx,edx
    movzx ebx,word [bgrxy+2]
    mul   ebx
    xor   edx,edx
    movzx ebx,word [scrxy+2]
    div   ebx
    add   eax,[esp]
    add   esp,4
   
  nostretch:
   
   
    cmp   [bgrdrawtype],dword 1
    jne   notiled
   
    mov  eax,[ypos]
    add  eax,ecx
    xor  edx,edx
    movzx ebx,word [bgrxy]
    div  ebx
    mov  eax,edx
    movzx  ebx,word [bgrxy+2]
    xor  edx,edx
    mul  ebx
    push eax
   
    mov  eax,[xpos]
    add  eax,[esp+8]
    movzx ebx,word [bgrxy+2]
    xor  edx,edx
    div  ebx
    mov  eax,edx
    add  eax,[esp]
    add  esp,4
   
  notiled:
   
   
    lea  ecx,[eax+eax*2]
   
    mov  eax,39
    mov  ebx,2
   
    int  0x40
   
  nobpix:
   
    pop  ecx
    pop  ebx
   
;    cmp  ecx,39
;    jbe  no_darken
;    sub  eax,0x101010
;    mov  eax,0x5566aa
;    and  eax,0x7f7f7f
;    mov  eax,0xffffff
;  no_darken:
   
    mov  edx,eax
    mov  eax,1
    int  0x40
   
    sub  ebx,1
    cmp  ebx,-1
    jnz  newb
    mov  ebx,41
   
    add  ecx,1
    cmp  ecx,51
    jnz  newb
   
    mov  eax,4
    mov  ebx,0*65536+42
    movzx ecx,byte [I_PARAM+8+11+11+7]
    shl   ecx,16
    add  ebx,ecx
    mov  ecx,0x000000         ; black shade of text
    mov  edx,labelt
    mov  esi,labellen-labelt
    add  ebx,1*65536+1
;    int  0x40
    mov  eax,4                ; white text
    sub  ebx,1*65536+1
    mov  ecx,0xffffff
    int  0x40
   
   
    mov  ebx,0                     ; draw icon
    mov  ecx,32
    mov  esi,I_END+256+54
   
  newpix:
   
    mov  edx,[esi]
    and  edx,0xffffff
   
    add  ebx,4
    mov  eax,1
    cmp  edx,0
    jz   nopix
;    int  0x40
  nopix:
    sub  ebx,4
   
    add  esi,3
   
    add  ebx,1
    cmp  ebx,32
    jne  newpix
    mov  ebx,0
   
    sub  ecx,1
    cmp  ecx,0
    jne  newpix
   
    ret
   
   
   
; DATA AREA
   
xpos        dd   15
ypos        dd  185
   
start_file  db  'SETUP      '
fname       db  'HD      BMP'
labelt:
            db  ' SETUP '
labellen:
   
bgrxy       dd  0x0
scrxy       dd  0x0
bgrdrawtype dd  0x0
   
            nop
   
I_PARAM:
   
I_END:
   
   
   
   