;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                   ;
;    MENUBAR for MenuetOS  - Compile with fasm      ;
;                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
use32
    org  0x0
             db   'MENUET01'  ; 8 byte id
             dd   0x01        ; required OS version
             dd   START       ; program start
             dd   I_END       ; program image size
             dd   0x11000      ; reguired amount of memory
             dd   0x10ff0      ; esp
             dd   0x0,0x0     ; param, icon
   
; ***************** start of program ******************
   
START:
   
    call set_variables        ; set variables once only
   
    mov  eax,60
    mov  ebx,1
    mov  ecx,message
    mov  edx,32
    int  0x40
   
    mov  eax,40
    mov  ebx,01000111b
    int  0x40
   
    mov  eax,6
    mov  ebx,icon_file
    xor  ecx,ecx
    mov  edx,-1
    mov  esi,tictable+1100
    int  0x40
   
    call draw_icons
   
start_after_minimize:
    call draw_window          ; draw menu bar
    call draw_info            ; draw time, date, cpu info
   
still:
    mov  eax,9                ; process info
    mov  ebx,tictable         ; pointer to 1024 bytes table
    int  0x40
    cmp  [real_num_pros],eax  ; has the number of process's changed
    je   no_change            ; if no skip code
    mov  [real_num_pros],eax  ; else save number of process's
    call draw_window          ; and update
   
no_change:
    cmp  [changed],1          ; has setup changed
    jne  not_changed          ; no then jump code
    call draw_window          ; update window
    call draw_info            ; update info
    mov  [changed],0          ; reset setup changed
   
not_changed:
    mov  eax,3                ; get system clock
    int  0x40                 ; do it
    cmp  eax,[ptime]          ; has the value changed
    jz   ml1                  ; no then dont redraw
    mov  [ptime],eax          ; save value
    call draw_info            ; redraw
   
ml1:
    mov  eax,23               ; wait for event with timeout
    mov  ebx,20               ; 20ms
    int  0x40                 ; do it
    cmp  eax,1                ; is it a redraw ?
    jz   red                  ; yes then go to code
    cmp  eax,2                ; is it a key ?
    jz   key                  ; yes then go to code
    cmp  eax,3                ; button ?
    jz   button               ; yes then go to code
    cmp  eax,7
    jne  still
    cmp  [ICON_ID],0xffff
    je   save_icons
    call update_icon
    jmp  still                ; start again
   
save_icons:
    mov  eax,33
    mov  ebx,icon_file
    mov  ecx,tictable+1100
    mov  edx,[file_length]
    xor  esi,esi
    int  0x40
    jmp  still
   
saved    db 'SAVED'
   
key:
    mov  eax,2                ; read a key and ignore
    int  0x40                 ; do it
    jmp  still                ; start again
   
red:
    call draw_window          ; update window
    call draw_info            ; update info
    jmp  still                ; start again
   
button:
    mov  eax,17               ; get pressed button id
    int  0x40                 ; do it
   
    cmp  ah,101               ; has minimize to left been pressed
    jne  no_left              ; no then jump over code
    mov  eax,51               ; create thread
    mov  ebx,1                ; 1 = create
    mov  ecx,left_button      ; thread enters at left button code
    mov  edx,0x6fff0          ; thread stack position
    int  0x40                 ; do it
    mov  eax,5                ; delay
    mov  ebx,20               ; 20/100 of a second
    int  0x40                 ; do it
    mov  eax,-1               ; exit
    int  0x40                 ; do it
   
no_left:
    cmp  ah,1                 ; is it MENUET button
    jnz  noselect             ; no then jump over code
    call menu_handler         ; handle menu
    jmp  still                ; start again
   
noselect:
    cmp  ah,2                 ; is it system setup button
    jnz  nosystem             ; no then jump over code
    cmp  [sys_stack],0x51000  ; is setup running
    jge  nosystem             ; if yes dont start another
    add  [sys_stack],0x1000   ; else add to stack
    mov  eax,51               ; create thread
    mov  ebx,1                ; create
    mov  ecx,sys_start        ; thread entry point
    mov  edx,[sys_stack]      ; thread stack position
    int  0x40                 ; do it
    jmp  still                ; start again
   
nosystem:
    cmp  ah,250               ; is it a button above prossess
    ja   nob1                 ; yes then jump over code
    cmp  ah,150               ; is it a button below prossess
    jb   nob1                 ; yes then jump over code
    movzx ecx,byte ah         ; move button number into ecx
    sub  ecx,150              ; remove our offset
    shl  ecx,2                ; multiply by 4 bytes per entry
    mov  eax,18               ; system service
    mov  ebx,3                ; activate window
    mov  ecx,[pros_list+ecx]  ; point to process list plus offset
    int  0x40                 ; do it
;    jmp  still                ; start again
   
nob1:
    jmp  still                ; start again
   
; **********  START OF SETUP THREAD  **********
   
sys_start:
   
    call sys_window           ; call draw setup window
   
sys_still:
    mov  eax,10               ; wait for event
    int  0x40                 ; do it
    cmp  eax,1                ; was it a window redraw
    je  sys_red               ; jump to redraw code
    cmp  eax,2                ; was it a key press
    je  sys_key               ; jump to key press code
    cmp  eax,3                ; was it a button press
    je  sys_button            ; jump to button press code
    jmp  sys_still            ; start again
   
sys_red:
    call sys_window           ; call draw setup window
    jmp  sys_still            ; start again
   
sys_key:
    mov  eax,2                ; get key
    int  0x40                 ; do it
    jmp  sys_still            ; start again
   
sys_button:
    mov  eax,17               ; get button id
    int  0x40                 ; do it
    cmp  ah,1                 ; was it the exit button
    jne  sys_noclose          ; no then jump over code
    mov  eax,33               ; save file
    mov  ebx,mbar_file        ; point to file name
    mov  ecx,fd               ; point to data
    mov  edx,fd_end-fd        ; how much data
    xor  esi,esi              ; esi = 0 create new
    int  0x40                 ; do it
    sub  [sys_stack],0x1000   ; remove from stack
    mov  eax,-1               ; close thread
    int  0x40                 ; do it
   
sys_noclose:
    cmp  ah,3                 ; was it time yes button
    jne  time_notyes          ; not this button try next
    mov  [show_time],1        ; save value for update
    mov  [changed],1          ; set changed flag
    jmp  sys_still            ; start again
   
time_notyes:
    cmp  ah,4                 ; was it time no button
    jne  time_notno           ; not this button try next
    mov  [show_time],0        ; save value for update
    mov  [changed],1          ; set changed flag
    jmp  sys_still            ; start again
   
time_notno:
    cmp  ah,5                 ; was it date yes button
    jne  date_notyes          ; not this button try next
    mov  [show_date],1        ; save value for update
    mov  [changed],1          ; set changed flag
    jmp  sys_still            ; start again
   
date_notyes:
    cmp  ah,6                 ; was it date no button
    jne  date_notno           ; not this button try next
    mov  [show_date],0        ; save value for update
    mov  [changed],1          ; set changed flag
    jmp  sys_still            ; start again
   
date_notno:
    cmp  ah,7                 ; 3D yes button
    jne  three_d_notyes       ; not this button try next
    mov  [show_three_d],1     ; save value for update
    mov  eax,48               ; define windows properties
    mov  ebx,1                ; button style
    mov  ecx,1                ; 3D buttons
    int  0x40                 ; do it
    mov  [changed],1          ; set changed flag
    jmp  sys_still            ; start again
   
three_d_notyes:
    cmp  ah,8                 ; 3D no button
    jne  three_d_notno        ; not this button try next
    mov  [show_three_d],0     ; save value for update
    mov  eax,48               ; define windows properties
    mov  ebx,1                ; button style
    mov  ecx,0                ; flat buttons
    int  0x40                 ; do it
    mov  [changed],1          ; set changed flag
;    jmp  sys_still            ; start again
   
three_d_notno:
    jmp  sys_still            ; start again
   
; ************  DRAW SETUP WINDOW  ************
   
sys_window:
   
    mov  eax,12               ; window redraw status
    mov  ebx,1                ; start of redraw
    int  0x40                 ; do it
   
    xor  eax,eax              ; define and draw window
    mov  ebx,[screenx]        ; get screen width
    shr  ebx,1                ; half it
    sub  ebx,100              ; subtract half window width
    shl  ebx,16               ; multiply by 65536
    mov  bx,200               ; add window width
    mov  ecx,[screeny]        ; get window height
    shr  ecx,1                ; half it
    sub  ecx,55               ; subtract half window height
    shl  ecx,16               ; multiply by 65536
    mov  cx,109               ; add window height
    mov  edx,[wcolor]         ; body colour
    add  edx,0x03000000       ; skin it
    mov  esi,0x8099bbff       ; grab bar colour
    mov  edi,0x0099bbee       ; frame colour
    int  0x40                 ; do it
   
    mov  eax,8                ; define button
    mov  ebx,129*65536+30     ; button x position and width
    mov  ecx,27*65536+20      ; button y position and height
    mov  edx,3                ; button id
    mov  esi,[wcolor]         ; button colour
    int  0x40                 ; do it
   
    mov  ebx,160*65536+30     ; button x position and width
    mov  edx,4                ; button id
    int  0x40                 ; do it
   
    mov  ebx,129*65536+30     ; button x position and width
    mov  ecx,53*65536+20      ; button y position and height
    mov  edx,5                ; button id
    int  0x40                 ; do it
   
    mov  ebx,160*65536+30     ; button x position and width
    mov  edx,6                ; button id
    int  0x40                 ; do it
   
    mov  ebx,129*65536+30     ; button x position and width
    mov  ecx,79*65536+20      ; button y position and height
    mov  edx,7                ; button id
    int  0x40                 ; do it
   
    mov  ebx,160*65536+30     ; button x position and width
    mov  edx,8                ; button id
    int  0x40                 ; do it
   
    mov  eax,4                ; write text to window
    mov  ebx,10*65536+34      ; text x and y position
    mov  ecx,0x00ffffff       ; text colour
    mov  edx,time_txt         ; point to text
    mov  esi,29               ; amount of characters
    int  0x40                 ; do it
   
    mov  ebx,10*65536+60      ; text x and y position
    mov  edx,date_txt         ; point to text
    int  0x40                 ; do it
   
    mov  ebx,10*65536+86      ; text x and y position
    mov  edx,three_d_txt      ; point to text
    int  0x40                 ; do it
   
    mov  eax,12               ; window redraw status
    mov  ebx,2                ; end of redraw
    int  0x40                 ; do it
   
    ret                       ; return
   
time_txt     db   'Display time         YES   NO'
date_txt     db   'Display date         YES   NO'
three_d_txt  db   'Display 3D           YES   NO'
sys_stack    dd   0x50000
   
; ***************  MENU HANDLER  **************
   
menu_handler:
   
    mov  ebx,[stcount]        ; get menu flag
    inc  ebx                  ; increment the flag
    and  ebx,1                ; and with 1
    mov  [stcount],ebx        ; save the flag
    cmp  ebx,1                ; is it set
    jnz  nostas               ; no then jump code
    mov  eax,19               ; start program
    mov  ebx,filename         ; point to filename
    int  0x40                 ; do it
    ret                       ; return
   
nostas:
    xor  ecx,ecx              ; zero count
   
newt:
    mov  eax,9                         ; process info
    mov  ebx,tictable                  ; pointer to 1024 buffer
    int  0x40                          ; do it
    cmp  ecx,eax                       ; have we done all process's
    jg   still_l                       ; yes then exit
    inc  ecx                           ; increment the count
    cmp  [tictable+10],dword 'MENU'    ; is it this
    jnz  newt2                         ; no then try next
    cmp  [tictable+14],dword '    '    ; is it this
    je   doit                          ; yes then terminate it
   
newt2:
    cmp  [tictable+10],dword 'SELE'    ; is it this
    jnz  newt                          ; no then try next process
    cmp  [tictable+14],word  'CT'      ; is it this
    jnz  newt                          ; no then try next process
   
doit:
    mov  eax,18               ; system service
    mov  ebx,2                ; force terminate
    dec  ecx                  ; frig factor
    int  0x40                 ; do it
    jmp  nostas               ; next one
   
still_l:
    ret                       ; return
   
; ************** draw minimised ***************
   
draw_small_left:
   
    pusha                     ; save registers
   
    mov  eax,12               ; window redraw status
    mov  ebx,1                ; start of redraw
    int  0x40                 ; do it
   
    xor  eax,eax              ; define and draw window
    mov  ebx,0*65536+56       ; x position and width
    mov  ecx,[screeny]        ; get screen height
    sub  ecx,20               ; subtract panel height
    shl  ecx,16               ; multiply by 65536
    mov  cx,20                ; add panel height
    int  0x40                 ; do it
   
    mov  eax,8                ; define LEFT button
    mov  ebx,47*65536+9       ; x position and width
    mov  ecx,0*65536+20       ; y position and height
    mov  edx,1                ; button id
    mov  esi,[wcolor]         ; get colour
    int  0x40                 ; do it
    mov  ebx,0*65536+46       ; x position and width
    mov  edx,2                ; button id
    int  0x40                 ; do it
    mov  eax,4                ; draw MENUET button shadow text
    mov  ebx,6*65536+6        ; x and y position
    mov  ecx,[wcolor]         ; get colour
    sub  ecx,0x303030         ; modify colour for shadow
    mov  edx,menuet_text      ; point to text
    mov  esi,6                ; number of characters
    int  0x40                 ; do it
    mov  ebx,7*65536+7        ; draw MENUET button text
    add  ecx,0x606060         ; modify colour for text
    int  0x40                 ; do it
    mov  ebx,50*65536+7       ; draw LEFT button text
    mov  edx,hidetext+1       ; point to text
    mov  esi,1                ; number of characters
    int  0x40                 ; do it
   
    mov  eax,12               ; window redraw status
    mov  ebx,2                ; end of redraw
    int  0x40                 ; do it
   
    popa                      ; restore registers
    ret                       ; return
   
   
small_draw dd 0x0
   
; ***************** left button code **************
   
left_button:
   
    mov  [small_draw],dword draw_small_left
   
    mov  eax,5                ; delay
    mov  ebx,20               ; 20 ms
    int  0x40                 ; do it
   
    call draw_small_left      ; draw the window
   
small_wait:
    mov  eax,10               ; wait for event
    int  0x40                 ; do it
   
    cmp  eax,1                ; is it a window redraw
    jne  no_win               ; no then jump code
    call [small_draw]         ; else redraw
    jmp  small_wait           ; start again
   
no_win:
    cmp  eax,2                ; is it a keypress
    jne  no_key               ; no then jump code
    mov  eax,2                ; get key
    int  0x40                 ; do it
    jmp  small_wait           ; start again
   
no_key:
    mov  eax,17               ; get pressed button id
    int  0x40                 ; do it
    cmp  ah,1                 ; was it the LEFT button
    jne  no_full              ; no then jump code
    mov  eax,51               ; create thread
    mov  ebx,1                ; create
    mov  ecx,start_after_minimize      ; thread entry position
    mov  edx,0x7fff0          ; thread stack position
    int  0x40                 ; do it
    mov  eax,5                ; delay
    mov  ebx,20               ; 20 ms
    int  0x40                 ; do it
    mov  eax,-1               ; terminate thread
    int  0x40                 ; do it
   
no_full:
    call menu_handler         ; handle menu
   
    jmp  small_wait           ; start again
   
; *************** set variables **************
   
set_variables:
   
    pusha                     ; save registers
   
    mov  eax,6                ; open file
    mov  ebx,mbar_file        ; point to file name
    xor  ecx,ecx              ; ecx = 0x00000000 reserved
    mov  edx,-1               ; edx = 0xffffffff reserved
    mov  esi,fd               ; point to data area
    int  0x40                 ; do it
   
    mov  eax,6                ; open file default color map
    mov  ebx,defcol           ; point to file name
    xor  ecx,ecx              ; ecx = 0
    mov  edx,-1               ; edx = 0xffffffff
    mov  esi,0x10000          ; buffer position
    int  0x40                 ; do it
   
    mov  eax,[0x10000+6*4]    ; get a colour at offset
    mov  [wcolor],eax         ; save it
   
    mov  eax,48               ; define window properties
    mov  ebx,2                ; window colours
    mov  ecx,0x10000          ; point to buffer
    mov  edx,10*4             ; number of bytes
    int  0x40                 ; do it
   
    mov  eax,14               ; get screen max x & max y
    int  0x40                 ; do it
    mov  ebx,eax              ; copy register
    shr  ebx,16               ; shift high bytes to low bytes
    mov  [screenx],ebx        ; save width
    and  eax,0x0000ffff       ; remove high bytes
    mov  [screeny],eax        ; save height
   
    sub  ebx,220              ; calc max number of buttons
    shr  ebx,6                ; divide by 64
    mov  [max_pros],ebx       ; save max process buttons
   
    mov  eax,48               ; define window properties
    mov  ebx,1                ; button style
    mov  ecx,1                ; 3D buttons
    cmp  [show_three_d],1     ; was saved value 3D buttons
    je   three_d_ok           ; yes then jump to int
    mov  ecx,0                ; no then set flat buttons
   
three_d_ok:
    int  0x40                 ; do it
   
    mov  eax,0x40404040       ; dividers for processes
    mov  edi,pros             ; point to pros
    mov  ecx,10               ; set count
    cld                       ;
    rep  stosd                ; repeat store
   
    popa                      ; restore registers
    ret                       ; return
   
defcol db 'DEFAULT.DTP'
   
   
; *****************************************************
; *********** WINDOW DEFINITIONS AND DRAW *************
; *****************************************************
   
draw_window:
   
    pusha                     ; save registers
   
    mov  eax,12               ; window redraw status
    mov  ebx,1                ; start of redraw
    int  0x40                 ; do it
   
    xor  eax,eax              ; define and draw window
    mov  ebx,[screenx]        ; width of window
    mov  ecx,[screeny]        ; screen height
    sub  cx,20                ; minus window height
    shl  ecx,16               ; multiplyed by 65536
    add  cx,20                ; height of window
    mov  esi,[wcolor]
    int  0x40                 ; do it
   
;    mov  eax,13
;    mov  ebx,[screenx]
;    mov  ecx,1*65536+19
;    mov  edx,[wcolor]
;    int  0x40
   
    mov  eax,8                ; define MENUET button
    mov  ebx,0*65536+46       ; x position and width
    mov  ecx,0*65536+20       ; y positon and height
    mov  edx,1                ; button id
    mov  esi,[wcolor]         ; give it a colour
    int  0x40                 ; do it
    mov  eax,4                ; draw MENUET text
    mov  ebx,6*65536+6        ; x and y position
    mov  ecx,[wcolor]         ; get a colour
    sub  ecx,0x303030         ; modify for shadow
    mov  edx,menuet_text      ; point to text
    mov  esi,6                ; number of characters
    int  0x40                 ; do it
    mov  ebx,7*65536+7        ; x and y position
    add  ecx,0x606060         ; modify colour for text
    int  0x40                 ; do it
   
    mov  eax,8                ; define SYSTEM button
    mov  ecx,0*65536+20       ; y position and height
    mov  ebx,[screenx]        ; screen width
    sub  ebx,45               ; width - 45 = none
    cmp  [show_time],1        ; is the time showing
    jne  nc_1                 ; no then jump code
    sub  ebx,49               ; width - 94 = time
    cmp  [show_date],1        ; is the date showing as well
    jne  nd_1                 ; no then jump code
    sub  ebx,71               ; width - 165 = time + date
    jmp  nd_1                 ; jump to define
   
nc_1:
    cmp  [show_date],1        ; is it just the date showing
    jne  nd_1                 ; no then jump code
    sub  ebx,79               ; 124 = date
   
nd_1:
    shl  ebx,16               ; multiply by 65536
    mov  bx,45                ; add button width
    mov  edx,2                ; button id
    mov  esi,[wcolor]         ; give it a colour
    int  0x40                 ; do it
   
    mov  eax,8                ; define LEFT button
    mov  ebx,47*65536+9       ; x position and width
    mov  edx,101              ; button id
    mov  esi,[wcolor]         ; give it a colour
    int  0x40                 ; do it
   
    mov  eax,4                ; draw LEFT ARROW text
    mov  ebx,49*65536+7       ; x and y position
    mov  ecx,[wcolor]         ; get colour
    add  ecx,0x303030         ; modify it
    mov  edx,hidetext         ; point to text
    mov  esi,1                ; number of characters
    int  0x40                 ; do it
   
    mov  eax,12               ; window redraw status
    mov  ebx,2                ; end of redraw
    int  0x40                 ; do it
   
    call draw_pros_buttons    ; update process buttons
   
    popa                      ; restore registers
    ret                       ; return
   
   
; ************* draw prossess buttons ************
   
max_pros        dd  0         ; calculated max number of buttons
num_pros        dd  0         ; number of process's we want
real_num_pros   dd  0         ; number of process's running
pros_list       dd  1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20
   
draw_pros_buttons:
   
    pusha                     ; save all registers
   
    mov  ebx,tictable         ; point to buffer
    xor  ecx,ecx              ; zero real process count
    xor  edx,edx              ; zero dislay process count
   
next_count:
    inc  ecx                           ; inc real process count
    mov  eax,9                         ; process info
    int  0x40                          ; do it
    cmp  [tictable+10],dword 'OS/I'    ; do we want this
    je   next_count                    ; no then next process
    cmp  [tictable+11],dword 'CON '    ; do we want this
    je   next_count                    ; no then next process
    cmp  [tictable+10],dword 'PANE'    ; do we want this
    je   next_count                    ; no then next process
    cmp  [tictable+11],dword 'ENU '    ; do we want this
    je   next_count                    ; no then next process
    cmp  [tictable+10],dword 'SELE'    ; do we want this
    je   next_count                    ; no then next process
    cmp  [tictable+10],dword 'END '    ; do we want this
    je   next_count                    ; no then next process
    cmp  [tictable+10],dword '    '    ; do we want this
    je   next_count                    ; no then next process
    mov  eax,[tictable+10]             ; get first part of text
    mov  [tictable+1024+(edx*8)],eax   ; save it to buffer
    mov  eax,[tictable+14]             ; get second part of text
    mov  [tictable+1024+(edx*8+4)],eax ; save it to buffer
    mov  [pros_list+edx*4],ecx         ; save process id
    inc  edx                           ; count process
    cmp  ecx,[real_num_pros]           ; have we done all process's
    jle  next_count                    ; no then next process
    dec  edx                           ; frig for over count
    mov  [num_pros],edx                ; save count
   
    cmp  [num_pros],0         ; are there any prossess running
    je   no_pros              ; no then jump code
   
    mov  eax,8                ; draw button
    mov  ebx,67*65536+60      ; x position of first button
    mov  ecx,0*65536+20       ; y position of all buttons
    mov  edx,150              ; id of first button
    mov  esi,[wcolor]         ; button colour
   
    mov  edi,[num_pros]       ; get the number of prossess
    cmp  edi,[max_pros]       ; is it greater than the max
    jle  next_pros            ; no then draw buttons
    mov  edi,[max_pros]       ; or make equal to max
    mov  [num_pros],edi       ; save for later
   
next_pros:
    int  0x40                 ; do it
   
    add  ebx,61*65536         ; x position for next button
    inc  edx                  ; next button id
    dec  edi                  ; decrement button count
    cmp  edi,0                ; have we drawn all buttons
    jne  next_pros            ; no then draw next button
   
    mov  eax,4                ; ICON text
    mov  ebx,74*65536+7       ; start position of text
    mov  ecx,[wcolor]         ; color of text
    add  ecx,0x303030         ; modified color
    mov  esi,8                ; 8 characters to draw
   
    xor  edi,edi
    mov  edx,tictable+1024-8  ; point at the text-8 bytes
   
next_pros_text:
    add  edx,8                ; move pointer 8 bytes
    int  0x40                 ; do it
    add  ebx,61*65536         ; move the text position
    inc  edi                  ; decrement text count
    cmp  edi,[num_pros]       ; have we drawn all the text
    jne  next_pros_text       ; no then do the next text
   
no_pros:
    popa                      ; restore all registers
    ret                       ; return
   
   
; ************** DRAW INFO *******************
   
draw_info:
   
    pusha                     ; save registers
   
sta:
    mov  eax,18 ; TSC / SEC   ; system service
    mov  ebx,5                ; time stamp counter / second
    int  0x40                 ; do it
    shr  eax,20               ; divide by 1048576
    push eax                  ; save 1
   
    mov  eax,18 ; IDLE / SEC  ; system service
    mov  ebx,4                ; idle clock cycles / second
    int  0x40                 ; do it
    shr  eax,20               ; divide by 1048576
    mov  ebx,100              ; use 100
    mul  ebx                  ; multiply eax
   
    pop  ebx                  ; restore 1
    inc  ebx                  ; add 1
    div  ebx                  ; devide by eax
    push eax                  ; save 2
   
    mov  esi,pros+1           ;
    mov  edi,pros             ;
    mov  ecx,55               ;
    rep  movsb                ;
   
    pop  eax                  ; restore 2
    mov  [pros+1],al          ;
   
    mov  ebx,[screenx]        ; get screen width
    sub  ebx,39               ; 39 = none
    cmp  [show_time],1        ; is the time visible
    jne  nc_2                 ; no then jump code
    sub  ebx,49               ; 88 = time
     cmp  [show_date],1        ; is the date visible aswell
     jne  nd_2                 ; no then skip code
    sub  ebx,71               ; 159 = time + date
    jmp  nd_2                 ; jump date only code
   
nc_2:
    cmp  [show_date],1        ; is the date visible
    jne  nd_2                 ; no then jump code
    sub  ebx,79               ; 118 = date
   
nd_2:
    shl  ebx,16               ; multiply by 65536
    mov  bx,34                ; add width
   
    mov  esi,pros             ;
    mov  edi,pros+1           ;
   
newpros:
    pusha                     ; save registers
    mov  eax,13               ; draw bar
    mov  ecx,3*65536+15       ; y position and height
    mov  edx,[wcolor]         ; get colour
    sub  edx,0x303030         ; modify it
    int  0x40                 ; do it
    popa                      ; restore registers
   
    inc  esi                  ;
    push ebx                  ; save 3
   
    xor  edx,edx              ; zero edx
    xor  eax,eax              ; zero eax
    mov  al,[esi]             ;
    mov  ebx,3                ;
    div  ebx                  ; devide by eax
    inc  eax                  ;
    mov  ecx,3*65536+15       ; y position and height
   
    pop  ebx                  ; restore 3
    mov  bx,ax                ; width
   
    mov  eax,13               ; draw bar
    mov  edx,[wcolor]         ; get colour
    int  0x40                 ; do it
   
    inc  bx                   ;
   
    cmp  esi,edi              ;
    jb  newpros               ;
   
overcpu:
    mov  eax,[screenx]        ;
    cmp  [show_time],1        ;
    jne  nc_3                 ;
    sub  eax,43               ; 43 = time
    cmp  [show_date],1        ;
    jne  nd_3                 ;
    sub  eax,71               ; 114 = time + date
    jmp  nd_3                 ;
   
nc_3:
    cmp  [show_date],1        ;
    jne  nd_3                 ;
    sub  eax,73               ; 73 = date
   
nd_3:
    shl  eax,16               ;
    mov  ebx,eax              ;
    cmp  [show_time],1        ;
    jne  nc_4                 ;
    mov  bx,39                ; 39 = time
    cmp  [show_date],1        ;
    jne  nd_4                 ;
    mov  bx,110               ; 110 = time + date
    jmp  nd_4                 ;
   
nc_4:
    cmp  [show_date],1        ;
    jne  nd_4                 ;
    mov  bx,69                ; 69 = date
   
nd_4:
    cmp  [show_time],1        ;
    jne  check_disp           ;
    jmp  show_bar             ;
   
check_disp:
    cmp  [show_date],1        ;
    jne  over_bar             ;
   
show_bar:
    mov  eax,13               ; draw bar on time
    mov  ecx,3*65536+15       ;
    mov  edx,[wcolor]         ;
    sub  edx,0x101010         ;
    int  0x40                 ;
   
over_bar:
    cmp  [show_time],1        ;
    jne  over_time            ;
   
    mov  eax,[ptime]          ; get time
   
    xor  edx,edx              ;
    xor  ecx,ecx              ;
    xor  ebx,ebx              ;
   
    mov  bl,al                ;
    shr  eax,8                ;
    mov  cl,al                ;
    shr  eax,8                ;
    mov  dl,al                ;
   
    push ebx                  ; ebx = h
    push ecx                  ; ecx = m
    push edx                  ; edx = s
   
    mov  ebx,[screenx]        ;
    cmp  [show_time],1        ;
    jne  nc_5                 ;
    sub  ebx,26               ; 26 = time
    cmp  [show_date],1        ;
    jne  nc_5                 ;
    sub  ebx,71               ; 97 = time + date
   
nc_5:
    shl  ebx,16               ;
    add  ebx,7                ;
   
    pop  edx                  ; __X__
    and  edx,1                ;
    add  edx,sec              ;
    mov  eax,4                ;
    mov  ecx,0xffffff         ;
    mov  esi,1                ;
    int  0x40                 ;
   
    pop  edx                  ; __:_X
    push edx                  ;
    and  edx,15               ;
    mov  eax,dword 0x00000004 ;
    add  ebx,12*65536         ;
    add  edx,text             ;
    mov  esi,1                ;
    int  0x40                 ;
   
    pop  edx                  ; __:X_
    shr  edx,4                ;
    and  edx,15               ;
    mov  eax,dword 0x00000004 ;
    sub  ebx,6*65536          ;
    add  edx,text             ;
    mov  esi,1                ;
    int  0x40                 ;
   
    pop  edx                  ; _X:__
    push edx                  ;
    and  edx,15               ;
    mov  eax,dword 0x00000004 ;
    sub  ebx,12*65536         ;
    add  edx,text             ;
    mov  esi,1                ;
    int  0x40                 ;
   
    pop  edx                  ; X_:__
    shr  edx,4                ;
    and  edx,15               ;
    mov  eax,dword 0x00000004 ;
    sub  ebx,6*65536          ;
    add  edx,text             ;
    mov  esi,1                ;
    int  0x40                 ;
   
over_time:
    cmp  [show_date],1        ;
    jne  over_date            ;
   
    mov  eax,29               ;
    int  0x40                 ;
    mov  edi,eax              ;
   
    mov  ebx,[screenx]        ;
    sub  ebx,68               ; date position
    shl  ebx,16               ;
    mov  bx,7                 ;
   
    mov  eax,47               ; day
    mov  edx,ebx              ;
    mov  ebx,0x020100         ;
    mov  ecx,edi              ;
    shr  ecx,16               ;
    mov  esi,0xffffff         ;
    int  0x40                 ;
   
    mov  eax,47               ; month
    add  edx,6*3*65536        ;
    mov  ecx,edi              ;
    shr  ecx,8                ;
    int  0x40                 ;
   
    mov  eax,47               ; year
    add  edx,18*65536         ;
    mov  ecx,edi              ;
    mov  ch,0x20              ;
    mov  ebx,0x040100         ;
    int  0x40                 ;
   
over_date:
    popa                      ;
    ret                       ;
   
; ************** DRAW INFO *******************
   
draw_icons:
   
    xor  edx,edx
    mov  eax,tictable+1100
    add  eax,16
   
count_next:
    cmp  [eax],dword 'END '
    je   count_end
    add  eax,42
    inc  edx
    jmp  count_next
   
count_end:
    mov  [num_icons],edx
    mov  [data_end],eax
    push eax
    push edx
    mov  esi,edx
    call DEC2ASC
    mov  dword [icon_num],ebx
    pop  edx
    pop  eax
    add  eax,4
    sub  eax,tictable+1100
    mov  [file_length],eax
    mov  ebx,program_name
    mov  ecx,tictable+1100
    add  ecx,16
   
next_icon:
    mov  eax,19
    int  0x40
    add  ecx,42
    dec  edx
    jz   all_done
    jmp  next_icon
   
all_done:
    ret
   
; ********  DECIMAL to ASCII  ****************
   
DEC2ASC:
   
    mov  ax,si
    mov  cx,1000
    xor  edx,edx
    idiv cx
    mov  bl,al
    add  bl,48
    shl  ebx,8
    mov  ax,dx
    mov  cx,100
    xor  edx,edx
    idiv cx
    mov  bl,al
    add  bl,48
    shl  ebx,8
    mov  ax,dx
    mov  cx,10
    xor  edx,edx
    idiv cx
    mov  bl,al
    add  bl,48
    shl  ebx,8
    mov  bl,dl
    add  bl,48
    bswap ebx
   
    ret
   
; ********  UPDATE ICON DATA  ****************
   
update_icon:
   
    mov  esi,[ICON_X]
    call DEC2ASC
    mov  eax,[ICON_ID]
    imul eax,42
    add  eax,tictable+1100
    add  eax,20
    mov  [eax],ebx
    mov  esi,[ICON_Y]
    call DEC2ASC
    mov  eax,[ICON_ID]
    imul eax,42
    add  eax,tictable+1100
    add  eax,24
    mov  [eax],ebx
    mov  dword [message+4],8
   
    ret
   
; ************  DATA  ************************
   
hidetext       db  0x11,0x10,0x1e
   
menuet_text    db  'Menuet'
   
filename       db  'MENU       '
   
program_name   db  'ICON       '
icon_file      db  'ICON    DAT'
file_length    dd  0
data_end       dd  0
num_icons      dd  0
   
icon_num       db  '0000'
icon_x         db  '0000'
icon_y         db  '0000'
icon_bmp       db  '        BMP'
icon_app       db  '           '
icon_text      db  '        '
icon_end       db  'END '
   
pros           db  '                                        '
               db  '                                        '
   
ptime          dd  0x0
   
text           db  '0123456789'
   
wcolor         dd  0x506070
   
sec            db  ': '
   
stcount        dd  0x0
   
screenx        dd  0x0
screeny        dd  0x0
   
mbar_file      db  'PANEL   DAT'
   
message:
               dd  0
               dd  8
ICON_pid       dd  0
ICON_len       dd  0
ICON_ID        dd  0
ICON_X         dd  0
ICON_Y         dd  0
   
fd:
show_time      dd  1          ; display the clock
show_date      dd  1          ; display the date
show_three_d   dd  1          ; display 3d
changed        dd  0          ; display changed ?
fd_end:
   
tictable:
   
I_END:
   
   
   
   
   
   
   
   
   
   